Module planner

Module planner 

Query planner interfaces.

Structs

UnsupportedQueryPlanner
A query planner that reports that planning is not implemented.

Traits

ExtensionPlanner
This trait exposes the ability to plan an ExecutionPlan out of a LogicalPlan.
PhysicalPlanner
Physical query planner that converts a LogicalPlan to an ExecutionPlan suitable for execution.
QueryPlanner
A planner that creates a physical plan for a query.