pub struct UnsupportedQueryPlanner;Expand description
A query planner that reports that planning is not implemented.
Session implementations that do not expose a query planner can return
this planner explicitly.