pub trait PhysicalPlanner: Send + Sync {
// Required methods
fn create_physical_plan<'life0, 'life1, 'life2, 'async_trait>(
&'life0 self,
logical_plan: &'life1 LogicalPlan,
session: &'life2 dyn Session,
) -> Pin<Box<dyn Future<Output = Result<Arc<dyn ExecutionPlan>>> + Send + 'async_trait>>
where Self: 'async_trait,
'life0: 'async_trait,
'life1: 'async_trait,
'life2: 'async_trait;
fn create_physical_expr(
&self,
expr: &Expr,
input_dfschema: &DFSchema,
session: &dyn Session,
planning_ctx: &PhysicalPlanningContext,
) -> Result<Arc<dyn PhysicalExpr>>;
}Expand description
Physical query planner that converts a LogicalPlan to an
ExecutionPlan suitable for execution.
Required Methods
Sourcefn create_physical_plan<'life0, 'life1, 'life2, 'async_trait>(
&'life0 self,
logical_plan: &'life1 LogicalPlan,
session: &'life2 dyn Session,
) -> Pin<Box<dyn Future<Output = Result<Arc<dyn ExecutionPlan>>> + Send + 'async_trait>>where
Self: 'async_trait,
'life0: 'async_trait,
'life1: 'async_trait,
'life2: 'async_trait,
fn create_physical_plan<'life0, 'life1, 'life2, 'async_trait>(
&'life0 self,
logical_plan: &'life1 LogicalPlan,
session: &'life2 dyn Session,
) -> Pin<Box<dyn Future<Output = Result<Arc<dyn ExecutionPlan>>> + Send + 'async_trait>>where
Self: 'async_trait,
'life0: 'async_trait,
'life1: 'async_trait,
'life2: 'async_trait,
Create a physical plan from a logical plan
Sourcefn create_physical_expr(
&self,
expr: &Expr,
input_dfschema: &DFSchema,
session: &dyn Session,
planning_ctx: &PhysicalPlanningContext,
) -> Result<Arc<dyn PhysicalExpr>>
fn create_physical_expr( &self, expr: &Expr, input_dfschema: &DFSchema, session: &dyn Session, planning_ctx: &PhysicalPlanningContext, ) -> Result<Arc<dyn PhysicalExpr>>
Create a physical expression from a logical expression suitable for evaluation
expr: the expression to convert
input_dfschema: the logical plan schema for evaluating expr
planning_ctx: the PhysicalPlanningContext used to resolve
Expr::ScalarSubquery nodes. During physical planning the planner
threads the context of the plan currently being converted to a physical
plan (for example into ExtensionPlanner::plan_extension, which
should forward it here). Callers creating physical expressions outside
of a plan should pass &PhysicalPlanningContext::default().
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".